﻿using System;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

namespace Head
    {
    
    public partial class Head : Form
        {

        // ******************************************* class constants

        const int   MAXIMUM_BUFFER = 256;
        const int   MAXIMUM_ENTRIES  = 8;
        const int   MAXIMUM_LINES = 10;
        const int   MAXIMUM_STRING = 10;

        enum ASCII_Codes {
            ASCII_NUL =  0,
            ASCII_SOH =  1,
            ASCII_STX =  2,
            ASCII_ETX =  3,
            ASCII_EOT =  4,
            ASCII_ENQ =  5,
            ASCII_ACK =  6,
            ASCII_BEL =  7,
            ASCII_BS  =  8,
            ASCII_HT  =  9,
            ASCII_LF  =  10,
            ASCII_VT  =  11,
            ASCII_FF  =  12,
            ASCII_CR  =  13,
            ASCII_SO  =  14,
            ASCII_SI  =  15,
            ASCII_DLE =  16,
            ASCII_DC1 =  17,
            ASCII_DC2 =  18,
            ASCII_DC3 =  19,
            ASCII_DC4 =  20,
            ASCII_NAK =  21,
            ASCII_SYN =  22,
            ASCII_ETB =  23,
            ASCII_CAN =  24,
            ASCII_EM  =  25,
            ASCII_SUB =  26,
            ASCII_ESC =  27,
            ASCII_FS  =  28,
            ASCII_GS  =  29,
            ASCII_RS  =  30,
            ASCII_US  =  31,
            ASCII_DEL =  127 }

        string [ ] ASCII_code_names = {
                                        // ASCII Control Codes
            "<NUL>", "<SOH>", "<STX>", "<ETX>", "<EOT>",    
            "<ENQ>", "<ACK>", "<BEL>", "<BS>",  "<HT>",    
            "<LF>",  "<VT>",  "<FF>",  "<CR>",  "<SO>",    
            "<SI>",  "<DLE>", "<DC1>", "<DC2>", "<DC3>",   
            "<DC4>", "<NAK>", "<SYN>", "<ETB>", "<CAN>",    
            "<EM>",  "<SUB>", "<ESC>", "<FS>",  "<GS>",    
            "<RS>",  "<US>",
                                        // ASCII Printing Characters
            " ",   "!",   "\"",  "#",   "$",    
            "%",   "&",   "'",   "(",   ")",    
            "*",   "+",   ",",   "-",   ".",    
            "/",   "0",   "1",   "2",   "3",    
            "4",   "5",   "6",   "7",   "8",    
            "9",   ":",   ";",   "<",   "=",    
            ">",   "?",   "@",   "A",   "B",    
            "C",   "D",   "E",   "F",   "G",    
            "H",   "I",   "J",   "K",   "L",    
            "M",   "N",   "O",   "P",   "Q",    
            "R",   "S",   "T",   "U",   "V",    
            "W",   "X",   "Y",   "Z",   "[",    
            "\\",   "]",   "^",   "_",   "`",    
            "a",   "b",   "c",   "d",   "e",    
            "f",   "g",   "h",   "i",   "j",    
            "k",   "l",   "m",   "n",   "o",    
            "p",   "q",   "r",   "s",   "t",    
            "u",   "v",   "w",   "x",   "y",    
            "z",   "{",   "|",   "}",   "~",    
                                        // ASCII DEL Code
            "<DEL>"
            } ;

        // ******************************************* class variables

        bool            hexadecimal = true;
        string          input_file_name = String.Empty;
        OpenFileDialog  input_file_OFD = null;
        bool            keep_on_top = false;
        string          last_initial_directory = String.Empty;
        bool            use_SP = false;

        // **************************************************** signon

        void signon ( )
            {
            string [ ]      pieces;
            StringBuilder   sb = new StringBuilder ( );

            pieces = this.ProductVersion.Split (
                          new char [ ] { '.' },
                          StringSplitOptions.RemoveEmptyEntries );

            sb.AppendFormat ( @"{0} V{1}.{2}",
                              this.ProductName,
                              pieces [ 0 ],
                              pieces [ 1 ] );

            this.Text = sb.ToString ( );
            this.Icon = Properties.Resources.HeadIcon;

            this.StartPosition = FormStartPosition.CenterScreen;
            }

        // ******************************************** byte_2_string

        string byte_2_string ( int  input_byte )
            {
            StringBuilder  sb = new StringBuilder ( );

            if ( ( input_byte == ( int ) ' ' ) && use_SP )
                {
                sb.Append ( "<SP>" );
                }
            else if ( input_byte <= ( int ) ASCII_Codes.ASCII_DEL )
                {
                sb.Append ( ASCII_code_names [ input_byte ] );
                }
                                        // generate <nnn> where nnn ==
                                        // value of input_byte
            else
                {
                int    byte_value = input_byte;
                int    value;
                
                value = byte_value / 100 ;
                sb.AppendFormat ( "<{0:D1}", value );
                byte_value = byte_value % 100 ;
                value = byte_value / 10 ;
                byte_value = byte_value % 10;
                sb.AppendFormat ( "{0:D1}{1:D1}>", 
                                  value, 
                                  byte_value );
                }
            
            return ( sb.ToString ( ) ) ;
            }

        // *************************************** display_input_file
        
        void display_input_file ( string  filename )
            {
            
            if ( String.IsNullOrEmpty ( filename ) )
                {
                throw ( new ArgumentNullException ( 
                            "filename",
                            "Filename must not be Null or empty" ) );
                }

            try 
                {
                StringBuilder  RTB_buffer = new StringBuilder ( );
                
                using ( FileStream fs = new FileStream ( 
                                                filename, 
                                                FileMode.Open,
                                                FileAccess.Read ) )
                    {
                    int             count = 0;
                    bool            done = false;
                    int [ ]         entry = new int [ 
                                                    MAXIMUM_ENTRIES ];
                    bool            eof_input = false;
                    int             file_byte;
                    int             index;
                    StringBuilder   line_buffer = new StringBuilder ( 
                                                    MAXIMUM_BUFFER );
                    int             line_count = 0;
                    int             starting_byte = 0;

                    while ( !( done || eof_input ) )
                        {
                        file_byte = fs.ReadByte ( );
                        eof_input = ( file_byte < 0 );
                        if ( !eof_input )
                            {
                            index = count;
                            count++;
                            entry [ index ] = file_byte;
                            }
                        if ( ( eof_input && ( count != 0 ) ) ||
                             ( count == MAXIMUM_ENTRIES ) )
                            {
                            line_buffer.Length = 0;
                            for ( int i = 0; 
                                    ( i < MAXIMUM_ENTRIES ); 
                                      i++ )
                                {
                                if ( i < count )
                                    {
                                    if ( hexadecimal )
                                        {
                                        line_buffer.AppendFormat ( 
                                            "{0:X2}  ",
                                            entry [ i ] );
                                        }
                                    else
                                        {
                                        line_buffer.AppendFormat ( 
                                            "{0,3:D} ",
                                            entry [ i ] );
                                        }
                                    }
                                else
                                    {
                                    line_buffer.Append ( "    " );
                                    }
                                }
                            line_buffer.AppendFormat ( "| {0,2:D} | ",
                                                      starting_byte );
                            starting_byte += MAXIMUM_ENTRIES;
                            
                            for ( int i = 0; ( i < count ); i++ )
                                {
                                line_buffer.Append ( byte_2_string ( 
                                                      entry [ i ] ) );
                                }
                            count = 0;
                            RTB_buffer.Append ( line_buffer );
                            RTB_buffer.Append ( Environment.NewLine );
                            line_count++;
                            done = ( line_count == MAXIMUM_LINES );
                            }
                        }
                    }

                file_view_RTB.Text = RTB_buffer.ToString ( );
                file_view_RTB.Visible = true;
                }
            
            catch ( Exception ex )
                {
                throw ( new ApplicationException ( ex.Message ) );
                }
            }

        // *************************************** initialize_tooltips

        void initialize_tooltips ( )
            {
            ToolTip  tooltip = new ToolTip ( );

            tooltip.InitialDelay = 100;
            tooltip.ReshowDelay = 100;
            tooltip.AutoPopDelay = 3000;

            tooltip.SetToolTip ( 
                hexadecimal_CHKBX, 
                "Check for hexadecimal; clear for decimal" );
            tooltip.SetToolTip ( 
                keep_on_top_CHKBX, 
                "Check to keep ViewFile on top" );
            tooltip.SetToolTip ( 
                use_sp_for_spaces_CHKBX, 
                "Check to replace spaces with <SP>" );
            tooltip.SetToolTip ( 
                exit_BUT, 
                "Click to exit ViewFile" );
            tooltip.SetToolTip ( 
                browse_BUT, 
                "Click to select file to display" );
            }

        // ****************************** initialize_global_variables
        
        void initialize_global_variables ( )
            {

            hexadecimal = true;
            input_file_name = String.Empty;
            keep_on_top = false;
            last_initial_directory = String.Empty;
            use_SP = false;

            initialize_tooltips ( );
            }

        // ******************************************* initialize_GUI

        void initialize_GUI ( )
            {

            exit_BUT.Visible = true;
            file_view_RTB.Visible = false;
            hexadecimal_CHKBX.Checked = hexadecimal;
            browse_BUT.Visible = true;
            input_file_LAB.Visible = true;
            input_file_TB.ForeColor = Color.Gray;
            input_file_TB.Text = "Browse to file you want displayed";
            input_file_TB.Visible = true;
            keep_on_top_CHKBX.Checked = keep_on_top;
            use_sp_for_spaces_CHKBX.Checked = use_SP;
            }

        // **************************************** OnApplicationExit

        void OnApplicationExit ( object     sender,
                                         EventArgs  e )
            {
            // this event handler is here in case it is needed
            }

        // ************************************* Head constructor
        
        public Head ( )
            {
            
            InitializeComponent ( );

            Application.ApplicationExit += new EventHandler (
                                   OnApplicationExit );

            signon ( );
            initialize_global_variables ( );
            initialize_GUI ( );
            }

        // ******************************* initialize_input_file_OFD

        void initialize_input_file_OFD ( )
            {

            input_file_OFD = new OpenFileDialog ( );

            input_file_OFD.CheckFileExists = true;
            input_file_OFD.CheckPathExists = true;
            input_file_OFD.FileName = "";
            input_file_OFD.Filter = "All Files|*.*";

            if ( String.IsNullOrEmpty ( last_initial_directory ) )
                {
                input_file_OFD.InitialDirectory = @"C:\";
                }
            else
                {
                input_file_OFD.InitialDirectory =
                    last_initial_directory;
                }

            input_file_OFD.Multiselect = false;
            input_file_OFD.RestoreDirectory = true;
            input_file_OFD.ShowHelp = false;
            input_file_OFD.SupportMultiDottedExtensions = false;
            input_file_OFD.Title = "Select Input File";
            }

        // ************************************************* BUT_Click
        
        void BUT_Click ( object     sender,
                         EventArgs  e )
            {
            Button  button = ( Button ) sender;
            string  name = button.Name;

            switch ( name )
                {
                case "browse_BUT":
                    initialize_GUI ( );
                    initialize_input_file_OFD ( );
                    if ( input_file_OFD.ShowDialog ( ) == 
                             DialogResult.Cancel )
                        {
                        
                        }
                    else if ( String.IsNullOrEmpty ( 
                                         input_file_OFD.FileName ) )
                        {
                        MessageBox.Show ( 
                            "An input file must be selected",
                            "Missing Input File" );
                        }
                    else
                        {
                        input_file_name = input_file_OFD.FileName;
                        input_file_TB.Text = 
                            Win32.truncate_path (
                                input_file_name,
                                input_file_TB.MaxLength );
                        input_file_TB.ForeColor = Color.Black;
                        
                        last_initial_directory = 
                            Path.GetDirectoryName ( input_file_name );

                        display_input_file ( input_file_name );
                        }
                    break;
                
                case "exit_BUT":
                    Application.Exit ( );
                    break;
                
                default:
                    throw new ApplicationException ( 
                        String.Format (
                            "{0} is not a recognized Button name",
                            name ) );
                }
            }

        // ************************************** CHKBX_CheckedChanged
        
        void CHKBX_CheckedChanged ( object    sender, 
                                    EventArgs e )
            {
            CheckBox    check_box = ( CheckBox ) sender;
            bool        is_checked = check_box.Checked;
            string      name = check_box.Name;
            
            switch ( name )
                {
                case "hexadecimal_CHKBX":
                    hexadecimal = is_checked;
                    display_input_file ( input_file_name );
                    break;
                    
                case "keep_on_top_CHKBX":
                    keep_on_top = is_checked;
                    this.TopMost = keep_on_top;
                    display_input_file ( input_file_name );
                    break;
                    
                case "use_sp_for_spaces_CHKBX":
                    use_SP = is_checked;
                    display_input_file ( input_file_name );
                    break;
                
                default:
                    throw new ApplicationException ( 
                        String.Format (
                            "{0} is not a recognized CheckBox name",
                            name ) );
                }
            }

        } // class Head

    // *************************************************** class Win32

    public class Win32
        {

        // http://www.csharp411.com/truncate-file-path-with-ellipsis/

        [DllImport ( "shlwapi.dll" )]
        static extern bool PathCompactPathEx (
                               [Out] StringBuilder sb,
                                      string path,
                                      int max_length,
                                      int flags );

        // ********************************************* truncate_path

        public static string truncate_path ( string path,
                                             int length )
            {
            StringBuilder sb = new StringBuilder ( );

            PathCompactPathEx ( sb, path, length, 0 );

            return ( sb.ToString ( ) );
            }

        } // public class Win32

    } // namespace Head
